"""
'Graduated Interval Recall'
see: http://www.google.com/search?q=graduated+interval+recall

This doesn't necessary follow Pimsleur's algorithm,
mostly because I couldn't find it documented. So
I made up my own method.  

The method involes dividing a list of 2^n cards
into ever smaller pieces.

For example, given 8 cards:

01234567

we want the following pattern:

0
1
01
2
3
23
0123
4
5
45
6
7
67
4567
01234567

reformatted, that becomes:
01012323012345456767456701234567
0 0     0               0
 1 1     1               1
    2 2   2               2
     3 3   3               3
            4 4     4       4
             5 5     5       5  
                6 6   6       6
                 7 7   7       7

so even though the spacing is larger for earlier
words, we generally do get the graduated pattern.

Also, for 2^n cards, we get n+1 repetitions of
each card. If each rep=2sec, then the ideal
cardset size will be one of:

   n=6 -> 12 minutes,  64 cards, 6 reps each
   n=7 -> 38 minutes, 128 cards, 7 reps each

The algorithm is just a recursive/fractal iteration...
"""
from __future__ import generators

def _ispoweroftwo(x):
    if x == 1:
        return 1
    else:
        return (x % 2 == 0) and _ispoweroftwo(x/2)

def _GIRkeygen(end, start=0):
    if end == 1:
        yield 0 + start
        return
    else:
        for x in _GIRkeygen(end/2, start): yield x
        for x in _GIRkeygen(end/2, start+end/2): yield x
    for x in range(0, end):
        yield x + start
   
def generate(cardlist):
    assert _ispoweroftwo(len(cardlist)), "len(cardlist) must be power of two"
    for index in _GIRkeygen(len(cardlist)):
        yield cardlist[index]

if __name__=="__main__":
    assert _ispoweroftwo(2)
    assert _ispoweroftwo(8)
    assert _ispoweroftwo(32)
    assert _ispoweroftwo(65536)
    assert not _ispoweroftwo(65524)
    goal = [0,1,0,1,2,3,2,3,0,1,2,3,4,5,4,5,6,7,6,7,4,5,6,7,0,1,2,3,4,5,6,7]
    real = [x for x in _GIRkeygen(8)]
    assert goal == real, real
    print "all tests passed"
