"""
This is the main learnfast program. 
"""
from __future__ import generators
import os, sys
import random, math, pygame
from pygame.locals import *
import pygame.mixer
import pygame.font
import lojfrq
import random
import girecall

### globals ########################################################


WINSIZE = (500, 400)
CENTER = (WINSIZE[0]/2,WINSIZE[1]/2)

MUSIC = "vivaldi_winter_largo.mid"

WHITE = 255, 255, 255
BLACK = 0,     0,   0
GREEN = 0,   127,   0

NUMCARDS = 64 #okay to change this, but it must be power of 2

TOPLINE, BOTTOMLINE = -1,1
SEEN = []

### defs ###########################################################

def promptColor(card):
    """
    New words get a green prompt.
    """
    color = (127,127, 127)
    if not card in SEEN:
        SEEN.append(card)
        color = GREEN
    return color


def getWordList():
    """
    returns a generator that produces the 128
    most popular gismu, shuffled timed in
    graduated intervals to aid short term memory.
    """
    gismu_defs = {}
    for line in open("gismu").readlines()[1:]:
        gismu = line[:5]
        rafsi = line[6:9], line[10:13], line[14:17]
        glico = line[19:38]
        hints = line[40:59]
        tanru = line[61:157]
        gismu_defs[gismu] = (gismu, glico.strip(), tanru.strip())
    words = [gismu_defs[g[1]] for g in lojfrq.rogismu[:NUMCARDS]]
    random.shuffle(words)
    return girecall.generate(words)


def escape():
    """
    returns 1 if the user escapes
    """
    for e in pygame.event.get():
        if e.type == QUIT:
            return 1
        elif e.type==KEYUP:
            if e.key== K_ESCAPE:
                return 1

def shortpause(screen):
    screen.fill(WHITE)
    pygame.display.flip()
    pygame.time.delay(4000)


def drawtext(screen, font, text, color, line):
    w,h = font.size(text)
    if line==TOPLINE:
        y = CENTER[1]-font.get_height()/2
    else:
        y = CENTER[1]+font.get_height()/2
    screen.blit(font.render(text, 0, color), (CENTER[0]-(w/2), y))


def init():
    pygame.mixer.pre_init(16000)
    pygame.init()
    screen = pygame.display.set_mode(WINSIZE)
    pygame.display.set_caption("learnfast")
    clock = pygame.time.Clock()
    if pygame.mixer.get_init():
        pygame.mixer.music.load("vivaldi_winter_largo.mid")
        pygame.mixer.music.play(-1)
    return screen,clock


def main():
    screen,clock = init()
    font = pygame.font.Font(None, 40)
    gen = getWordList()
    
    done = 0
    tick, tock = BLACK, WHITE
    this, next = tick, tock
    shortpause(screen)
    try:
        while not done:
            done = escape()
            clock.tick(1)
            this, next = next, this
            if this == tock:
                gismu, glico, tanru = gen.next()
            else:
                pygame.mixer.Sound("wave" + os.sep + gismu + ".wav").play()
            screen.fill(WHITE)
            drawtext(screen, font, gismu, this, TOPLINE)
            drawtext(screen, font, glico, promptColor(gismu), BOTTOMLINE)
            pygame.display.flip()
    except StopIteration:
        shortpause(screen)

if __name__=="__main__":
    main()
