"""
This is the program used to
collect the sound samples.

It requires the Snack sound toolkit for python.
NOTE: it tends to lock up on my win32 machine,
but only after recording the files
"""
import Tkinter
from tkSnack import *
import lojfrq

# SAVEDIR: where to save files. trailing slash required.
SAVEDIR = "./wave/"

class WaveRecorder(Tkinter.Frame):
    """
    A simple 'tape recorder' widget.
    """

    def __init__(self, parent=None, sound=None, *args, **kw):
        Tkinter.Frame.__init__(self)
        self.words = [row[0] for row in lojfrq.rogismu[:64]]

        ## make corrections with:
        ## self.words = "djica zdani xamgu".split()
        self.words = ["preti"]

        self.word = Tkinter.Label(self, text='hello', font=('Arial Black', 15))
        self.word.pack(side='top')
        Tkinter.Button(self, text='go', command=self.tick).pack(side='bottom')

    def tick(self):
        if self.words:
            self.prompt(self.words.pop())
        else:
            self.word['text'] = ''

    def prompt(self, word):
        self.word['text'] = word
        self.sound = Sound()
        self.sound.record()
        self.after(1500, self.save)

    def save(self):
        ## stop recording
        self.sound.stop() 
        
        ## trim the silence from around the sound:
        pad = 15 # number of 10-millisecond chunks to keep 
        pitch = self.sound.pitch()
        for start in range(len(pitch)):
            if pitch[start] > 0: break
        for end in range(len(pitch)-1,0,-1):
            if pitch[end] > 0: break
        start = max(0, start -pad)
        end = min(len(pitch), end+pad)
        length = self.sound.length()
        a = int(float(start)/len(pitch) * length)
        b = int(float(end)/len(pitch)  * length)
        self.sound.crop(start=a, end=b)

        ## now save it
        self.sound.write(SAVEDIR + self.word['text'] + ".wav")

        ## and show the next word:
        self.tick()


if __name__=="__main__":
    import sys
    if not "dos" in sys.argv:
        print "type 'dos' after the name of this file."
        print "it doesn't do anything but it keeps me from"
        print "trying to run it while in emacs. :)"
        sys.exit()
    root = Tkinter.Tk()
    initializeSnack(root)
    frame = WaveRecorder(root)
    frame.pack(expand=0)
    root.mainloop()
